import pygame
import json
import os
import sys
import time
import logging
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import cv2

pygame.font.init()

WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
DARK_GRAY = (30, 30, 60)
LIGHT_GRAY = (150, 150, 150)
RED = (200, 50, 0)
CCC_GREEN = (0, 163, 86)
CCC_BLUE = (0, 132, 176)
BG_COLOR = BLACK # (59, 0, 112)

class LoggingEventHandler(FileSystemEventHandler):
    """Logs all the events captured."""

    def __init__(self, callback, *args, **kwargs):
        super(LoggingEventHandler, self).__init__(*args, **kwargs)
        self.callback = callback

    def on_moved(self, event):
        super(LoggingEventHandler, self).on_moved(event)

        what = 'directory' if event.is_directory else 'file'
        logging.info("Moved %s: from %s to %s", what, event.src_path,
                     event.dest_path)

    def on_created(self, event):
        super(LoggingEventHandler, self).on_created(event)

        what = 'directory' if event.is_directory else 'file'
        logging.info("Created %s: %s", what, event.src_path)

    def on_deleted(self, event):
        super(LoggingEventHandler, self).on_deleted(event)

        what = 'directory' if event.is_directory else 'file'
        logging.info("Deleted %s: %s", what, event.src_path)

    def on_modified(self, event):
        super(LoggingEventHandler, self).on_modified(event)

        # if not event.is_directory and event.src_path == "current_round.json":
        self.callback()

        what = 'directory' if event.is_directory else 'file'
        logging.info("Modified %s: %s", what, event.src_path)


class Live(object):


    def __init__(self, window_width=-1):

        self.window_width = window_width

        self.side_width = 768
        self.height = 768

        self.main_width = 2560
        
        self.left_screen = pygame.surface.Surface((self.side_width, self.height))
        self.main_screen = pygame.surface.Surface((self.main_width, self.height))
        self.right_screen = pygame.surface.Surface((self.side_width, self.height))

        #self.display = pygame.display.set_mode((self.side_width + self.side_width + self.main_width, self.height))

        cv2.namedWindow("output")

        _font = "PressStart2P-Regular.ttf"
        self.font_xxl = pygame.font.Font(_font, 150)
        self.font_xl = pygame.font.Font(_font, 75)
        self.font_l = pygame.font.Font(_font, 40)
        self.font_m = pygame.font.Font(_font, 30)
        self.font_s = pygame.font.Font(_font, 20)

        # intro_screen
        # player_select
        # running_round
        # final_screen
        self.state = "intro_screen"

        self.current_player = -1 # -1 No Team, 0 Left Team, 1 Right Team

        self.team_names = ["", ""]
        self.team_points = [0, 0]
        self.team_fails = [0, 0]

        self.current_round = {}

        self.not_animated = [1, 1, 1, 1, 1]
        self.vis_questions = [0, 0, 0, 0, 0]

        ## PRE RENDERS '_pr'

        self.debug_side_pr = pygame.surface.Surface((1080, 1080))
        self.debug_side_pr.fill((0, 0, 30))
        self.debug_middle_pr = pygame.surface.Surface((1920, 1080))
        self.debug_middle_pr.fill((30, 0, 0))

        self.header_pr = self.font_xl.render("Chaos-Familien-Duell", True, CCC_BLUE, BG_COLOR)
        self.slogan_pr = self.font_m.render("Wir haben 1509 HackerInnen gefragt...", True, WHITE, BG_COLOR)
        # self.final_slogan_pr = self.font_m.render("Vielen Dank an watterott.com für die tollen Preise!", True, WHITE, BG_COLOR)
        self.final_slogan_pr = self.font_m.render("Die Entwicklung dieser Software wurde freundlicherweise unterstützt von", True, WHITE, BG_COLOR)
        self.koffein = self.font_xxl.render("KOFFEIN", True, WHITE, BG_COLOR)


        self.x_pr = self.font_xxl.render("X", True, RED, BG_COLOR)
        self.x_small_pr = self.font_m.render("X", True, WHITE, DARK_GRAY)

        self.chaos_west_pr = pygame.image.load("ChaosWest.png")
        self.watterott_pr = pygame.image.load("watterott.png")

        self.chaos_west_pr = pygame.transform.scale(self.chaos_west_pr, (400, 500))
        self.chaos_west_text_pr = pygame.image.load("chaos_west_text.jpg")
        self.gray_box_pr = pygame.surface.Surface((1200, 60))
        self.gray_box_pr.fill(DARK_GRAY)

        self.team_left_pr = None
        self.team_right_pr = None
        self.question_pr = None

        self.logo_left_pr = None
        self.logo_right_pr = None
        self.answers_pr = [None, None, None, None, None]
        self.answered_pr = [None, None, None, None, None]

        self.file_changed()


        logging.basicConfig(level=logging.INFO,
                            format='%(asctime)s - %(message)s',
                            datefmt='%Y-%m-%d %H:%M:%S')
        path = '.'
        event_handler = LoggingEventHandler(self.file_changed)
        observer = Observer()
        observer.schedule(event_handler, path, recursive=True)
        observer.start()


    def file_changed(self):
        with open("current_round.json", 'r') as target:
            self.current_round = json.load(target)

        state = self.current_round["state"]

        self.team_names = self.current_round["current_teams"]
        self.team_points = self.current_round["current_team_points"]
        self.team_fails = self.current_round["current_team_fails"]
        self.round_points = self.current_round["points"]
        self.current_player = -1

        if len(self.team_names[0]) > 30:
            _font_left = self.font_s
        else:
            _font_left = self.font_l

        if len(self.team_names[1]) > 30:
            _font_right = self.font_s
        else:
            _font_right = self.font_l
        
        self.team_left_pr = _font_left.render(self.team_names[0], True, WHITE, BG_COLOR)
        self.team_right_pr = _font_right.render(self.team_names[1], True, WHITE, BG_COLOR)
        self.question_pr = self.font_l.render(self.current_round["question"], True, CCC_BLUE, BG_COLOR)

        # _p1 = "../static/teamlogos" + str(self.current_round["current_teams_id"][0]) + ".png"
        # _p2 = "data/logo_" + str(self.current_round["current_teams_id"][1]) + ".png"

        _p1 = "../" + self.current_round["current_team_logos"][0]
        _p2 = "../" + self.current_round["current_team_logos"][1]

        if os.path.exists(_p1):
            self.logo_left_pr = pygame.image.load(_p1)
        if os.path.exists(_p2):
            self.logo_right_pr = pygame.image.load(_p2)

        for i in range(0, self.current_round["answer_amount"]):
            self.vis_questions[i] = self.current_round["answers"][i]["visible"]
            self.answers_pr[i] = self.font_m.render(self.current_round["answers"][i]["answer"], True, WHITE, DARK_GRAY)
            self.answered_pr[i] = self.font_m.render(str(self.current_round["answers"][i]["numberOfPeople"]), True, WHITE, DARK_GRAY)

        self.state = state

    def update(self):

        if self.state == "intro_screen":
            pass
        elif self.state == "player_select": 
            pass # TODO: Read input from hardware
        elif self.state == "running_round":
            pass # ....
        elif self.state == "final_screen":
            pass # ...

    def draw(self):

        self.main_screen.fill(BG_COLOR)
        self.left_screen.fill(BG_COLOR)
        self.right_screen.fill(BG_COLOR)
        #self.left_screen.fill((255, 0, 0))
        #self.right_screen.fill((0, 0, 255))

        if self.state == "intro_screen":
            self.main_screen.blit(self.chaos_west_text_pr, (((self.main_width / 2) - (self.chaos_west_text_pr.get_width() / 2)), 200))
            self.main_screen.blit(self.header_pr, ((self.main_width / 2) - (self.header_pr.get_width() / 2), 20))
            self.main_screen.blit(self.slogan_pr, ((self.main_width / 2) - (self.slogan_pr.get_width() / 2), 150))
            self.left_screen.blit(self.chaos_west_pr, ((self.side_width / 2) - (self.chaos_west_pr.get_width() / 2), (self.height / 2) - (self.chaos_west_pr.get_height() / 2)))
            self.right_screen.blit(self.chaos_west_pr, ((self.side_width / 2) - (self.chaos_west_pr.get_width() / 2), (self.height / 2) - (self.chaos_west_pr.get_height() / 2)))

        elif self.state == "player_select": 
            self.main_screen.blit(self.header_pr, ((self.main_width / 2) - (self.header_pr.get_width() / 2), 20))

            self.main_screen.blit(self.question_pr, (530, 250))
            
            for i in range(0, self.current_round["answer_amount"]):
                self.main_screen.blit(self.gray_box_pr, ((self.main_width / 2) - (self.gray_box_pr.get_width() / 2), 350 + (i * 70)))

                if not self.vis_questions[i] and self.not_animated[i] <= 0:
                    self.not_animated[i] = 1

                if self.vis_questions[i]:
                    self.main_screen.blit(self.answers_pr[i], (700, 364 + (i * 70)))
                    self.main_screen.blit(self.answered_pr[i], (1850 - self.answered_pr[i].get_width(), 363 + (i * 70)))

                if self.vis_questions[i] and self.not_animated[i] > 0:
                    self.not_animated[i] -= 0.2
                    if (self.not_animated[i] <= 0):
                        self.not_animated[i] = 0.001
                    _tmp_sur = pygame.surface.Surface((self.gray_box_pr.get_width() * self.not_animated[i], self.gray_box_pr.get_height()))
                    _tmp_sur.fill(DARK_GRAY)
                    
                    self.main_screen.blit(_tmp_sur, ((self.main_width / 2) - (self.gray_box_pr.get_width() / 2), 350 + (i * 70)))

            _points_left = self.font_xxl.render(str(self.team_points[0]), True, CCC_GREEN, BG_COLOR)
            _points_right = self.font_xxl.render(str(self.team_points[1]), True, CCC_GREEN, BG_COLOR)

            self.main_screen.blit(_points_left, (0, (self.height/2) - (_points_left.get_height() / 2)))
            self.main_screen.blit(_points_right, ((self.main_width - 0 - _points_right.get_width()), (self.height/2) - (_points_left.get_height() / 2)))

            _round_points = self.font_xl.render(str(self.round_points), True, CCC_GREEN, BG_COLOR)
            self.main_screen.blit(_round_points, (1200, 140))

            self.left_screen.blit(self.team_left_pr, ((self.side_width/2) - (self.team_left_pr.get_width()/2), 60))
            self.left_screen.blit(self.logo_left_pr, ((self.side_width/2) - (self.logo_left_pr.get_width()/2), self.height - self.logo_left_pr.get_height() - 20))
            self.right_screen.blit(self.team_right_pr, ((self.side_width/2) - (self.team_right_pr.get_width()/2), 60))
            self.right_screen.blit(self.logo_right_pr, ((self.side_width/2) - (self.logo_right_pr.get_width()/2), self.height - self.logo_right_pr.get_height() - 20))

            for i in range(0, self.team_fails[0]):
                self.main_screen.blit(self.x_pr, (60 + (i * 175), 600))
            for i in range(0, self.team_fails[1]):
                self.main_screen.blit(self.x_pr, (2530 - ((i+1) * 175), 600))


        elif self.state == "final_screen":
            # self.main_screen.blit(self.watterott_pr, ((self.main_width / 2) - (self.watterott_pr.get_width() / 2), -100))
            self.main_screen.blit(self.final_slogan_pr, ((self.main_width / 2) - (self.final_slogan_pr.get_width() / 2), 150))
            self.main_screen.blit(self.koffein, ((self.main_width / 2) - (self.koffein.get_width() / 2), 250))
            # self.main_screen.blit(self.header_pr, ((self.main_width / 2) - (self.header_pr.get_width() / 2), 20))

        self.output()

    def output(self):


        _tmp = pygame.surface.Surface((4096, 768))
        _tmp.blit(self.left_screen, (0, 0))
        _tmp.blit(self.main_screen, (self.side_width, 0))
        _tmp.blit(self.right_screen, (self.side_width + self.main_width, 0))

        out = pygame.surfarray.array3d(_tmp)
        out = out.swapaxes(0, 1)
        out = cv2.cvtColor(out, cv2.COLOR_BGR2RGB)

        if self.window_width != -1:
            out = cv2.resize(out, (self.window_width, int(self.height / ((self.side_width * 2 + self.main_width) / self.window_width))))

        cv2.imshow("output", out)

        key = cv2.waitKey(1)
        if key == 27:
            exit(0)
        if key == ord("f"):
            cv2.setWindowProperty("output", cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
