from peewee import IntegerField, CharField, ForeignKeyField

from model import BaseModel
from model import Team
from model import Question

class Game(BaseModel):

    name = CharField(primary_key=True)
    team_left = ForeignKeyField(Team)
    team_right = ForeignKeyField(Team)
    points_left = IntegerField(default=0)
    points_right = IntegerField(default=0)

    round_counter = IntegerField(default=0)

    def add_round(self, round):
        self.rounds.append(round)

    def remove_round(self, round):
        if round in self.rounds:
            self.rounds.remove(round)
            self.round_counter -= 1

    def jump_to_round(self, round):
        for i in range(len(self.rounds)):
            if self.rounds[i] == round:
                self.round_counter = i
                return self.get_current_round()
        return None


    def next_round(self):
        self.round_counter += 1
        return self.get_current_round()

    def get_all_rounds(self):
        return self.rounds.values()

    def get_current_round(self):
        if self.round_counter < len(self.rounds):
            return self.rounds[self.round_counter]
        return None



class Round(BaseModel):

    game = ForeignKeyField(Game, backref="rounds")
    points = IntegerField(default=0)
    question = ForeignKeyField(Question)
    error_left = IntegerField(default=0)
    error_right = IntegerField(default=0)

    def show_answer(self, id):
        answer = self.question.get_answer_by_id(id)
        if answer.hidden:
            self.points += answer.points
            answer.show()
            answer.save()
            self.save()
        return answer

    def hide_answer(self, id):
        answer = self.question.get_answer_by_id(id)
        if not answer.hidden:
            self.points -= answer.points
            answer.hide()
            answer.save()
            self.save()
        return answer


    @property
    def id(self):
        self.question.id

    @id.setter
    def id(self, id):
        pass

    @id.deleter
    def id(self):
        pass