from flask import request, Blueprint, jsonify, make_response
# from rest import rest_game
from rest import API_URI_PREFIX
from playhouse.shortcuts import model_to_dict
import json

from model import Game, Team, Round, Question

rest_game = Blueprint(__name__, "rest_game")

@rest_game.route(API_URI_PREFIX +"/game", methods=["POST"])
def post_game():
    if request.is_json:
        values = request.get_json()
    else:
        values = request.values
    teamname_left = values.get("team_left")
    teamname_right = values.get("team_right")

    if teamname_right is None or teamname_left is None:
        return make_response(jsonify("Teamnames are required"), 400)

    team_left = Team.get_or_none(name=teamname_left)
    team_right = Team.get_or_none(name=teamname_right)

    if team_right is None or team_left is None:
        return make_response(jsonify("Invalid teamname"), 400)

    gamename = values.get("name")
    if gamename == "" or gamename is None:
        gamename = teamname_left + "_" + teamname_right

    game = Game.get_or_none(name=gamename)
    if game is not None:
        return make_response(jsonify("Game already exists"), 400)
    
    try:
        game = Game.create(name=gamename, team_left=team_left, team_right=team_right)
    except:
        return make_response(jsonify("Error while creating game instance"), 500)

    return get_game(gamename)

@rest_game.route(API_URI_PREFIX +"/game/<name>", methods=["GET"])
def get_game(name):
    game = Game.get_or_none(name=name)
    if game is None:
        return make_response(jsonify("Invalid gamename"), 400)
    return make_response(jsonify(model_to_dict(game)), 200)

@rest_game.route(API_URI_PREFIX +"/allgames", methods=["GET"])
def get_all_games():
    game_list = Game.select()
    game_json = []
    for game in game_list:
        game_json.append(model_to_dict(game))
    return make_response(jsonify(game_json, 200))

@rest_game.route(API_URI_PREFIX +"/game/<name>", methods=["DELETE"])
def delete_game(name):
    if name is None or name == "":
        return make_response(jsonify("Gamename is required"), 400)

    game = Game.get_or_none(name=name)
    if game is None:
        return make_response(jsonify("No game by that name"), 400)

    try:
        game.delete_instance()
    except:
        return make_response(jsonify("Error occurred while attempting to remove record"), 500)

    return make_response("", 200)

@rest_game.route(API_URI_PREFIX +"/game/<name>", methods=["PUT"])
def put_game(name):
    if request.is_json:
        values = request.get_json()
    else:
        values = request.values

    game = Game.get_or_none(name=name)
    if game is None:
        return make_response(jsonify("Invalid gamename"), 400)

    if values.get("round_counter") is not None:
        return make_response(jsonify("round_counter cannot be set externaly"), 400)

    team_left = values.get("team_left")
    team_right = values.get("team_right")
    points_left = values.get("points_left")
    points_right = values.get("points_right")

    if team_left is not None:
        if type(team_left) is dict:
            team_left = team_left["name"]
        team = Team.get_or_none(name=team_left)
        if team is None:
            return make_response(jsonify("Invalid teamname: left"), 400)
        game.team_left = team
        
    if team_right is not None:
        if type(team_right) is dict:
            team_right = team_right["name"]
        team = Team.get_or_none(name=team_right)
        if team is None:
            return make_response(jsonify("Invalid teamname: right"), 400)
        game.team_right = team

    if points_left is not None:
        game.points_left = points_left

    if points_right is not None:
        game.ponts_right = points_right

    game.save()

    return get_game(name)

    @rest_game.route(API_URI_PREFIX + "/round/<question_id>", methods=["GET"])
    def get_round(question_id):
        question = Question.get_or_none(uuid=question_id)
        if question is None:
            return make_response(jsonify("No question by that id"), 400)

        round = Round.get_or_none(question=Question)
        if round is None:
            return make_response(jsonify("The is no round for the given question"), 400)

        return make_response(jsonify(model_to_dict(round)), 200)
