from flask import request, Blueprint, jsonify, make_response
from playhouse.shortcuts import model_to_dict

from rest import API_URI_PREFIX
from model import Team

rest_team = Blueprint(__name__, "rest_team")

@rest_team.route(API_URI_PREFIX + "/team/<teamname>", methods=["GET"])
def get_team(teamname):
    team = Team.get_or_none(name=teamname)
    if team is None:
        return make_response(jsonify("Invalid teamname"), 400)

    return make_response(jsonify(model_to_dict(team)), 200)

@rest_team.route(API_URI_PREFIX + "/allteams", methods=["GET"])
def get_all_teams():
    team_list = Team.select()
    team_json = []
    for team in team_list:
        team_json.append(model_to_dict(team))

    return make_response(jsonify(team_json), 200)

@rest_team.route(API_URI_PREFIX + "/team/<teamname>", methods=["DELETE"])
def delete_team(teamname):
    team = Team.get_or_none(name=teamname)
    if team is None:
        return make_response(jsonify("No team by that name"), 400)

    try:
        team.delete_instance()
    except:
        return make_response(jsonify("Error occurred while attempting to remove record"), 500)

    return make_response("", 200)

@rest_team.route(API_URI_PREFIX + "/team", methods=["POST"])
def post_team():
    if request.is_json:
        values = request.get_json()
    else:
        values = request.values
    name = values.get("name")
    logo_path = values.get("logo_path")

    if name is None or name == "":
        return make_response(jsonify("name is required"), 400)

    if logo_path is None or logo_path == "":
        return make_response(jsonify("Logopath is required"), 400)

    team = Team.get_or_none(name=name)
    if team is not None:
        return make_response(jsonify("There already exists a team by that name"), 400)

    Team.create(name=name, logo_path=logo_path)

    return get_team(name)

@rest_team.route(API_URI_PREFIX + "/team/<name>", methods=["PUT"])
def put_team(name):
    if request.is_json:
        values = request.get_json()
    else:
        values = request.values

    team = Team.get_or_none(name=name)
    if team is None:
        return make_response(jsonify("No team by that name"), 400)

    if values.get("name") is not None:
        return make_response(json("teamname cannot be changed"), 400)

    logo_path = values.get("logo_path")
    if logo_path is not None:
        team.logo_path = logo_path
    
    team.save()

    return make_response(jsonify(model_to_dict(team)), 200)